from django.db import models
from wagtail.admin.panels import (
    TabbedInterface, ObjectList, FieldPanel, InlinePanel, MultiFieldPanel, FieldRowPanel, HelpPanel,
    PublishingPanel,PageChooserPanel
)
from django.http import HttpResponseRedirect

from wagtail.fields import StreamField, RichTextField
from wagtail.models import Page,  PreviewableMixin, Orderable, PageManager
from wagtail.search import index
from wagtail.snippets.models import register_snippet, register_deferred_snippets
from modelcluster.models import ClusterableModel
from modelcluster.fields import ParentalKey
from modelcluster.contrib.taggit import ClusterTaggableManager
from taggit.models import TagBase, GenericTaggedItemBase, ItemBase

from wagtail_trash.managers import TrashManager
from wagtailmodelchooser import register_model_chooser
from wagtail_color_panel.fields import ColorField
from wagtail_color_panel.edit_handlers import NativeColorPanel
from wagtailmetadata.models import MetadataPageMixin
from wagtail.blocks import *
from wagtail.contrib.routable_page.models import RoutablePageMixin, path, re_path
from core.models import CustomTag

from .blocks import SectionBlock, RowBlock
from core.blocks import CarouselBlock,HeadingBlock

from core.common import MOB_MENU_POS_LIST
from core.abstract import AbstractCarousel, AbstractCustomColor,AbstractHero
from wagtail.images.blocks import ImageBlock
from core.models import SocialPlatform
from ecom.models import CategoryPage, ProductPage, ProductBrandPage, Category, Brands,ProductVariantAttributeValue
from ecom.panels import SubcategoryChooser
from hitcount.models import HitCountMixin, HitCount
from django.db.models import Q
from django import forms
from wagtail_color_panel.blocks import NativeColorBlock
from ecom.models import App,AppItems,ProductVariantPage

# from wagtail_newsletter.models import NewsletterPageMixin
from django.shortcuts import render

from core.blocks import SectionBlock as coreSection
from core.blocks import RowBlock as rowSection


from django.core.paginator import EmptyPage, PageNotAnInteger, Paginator
from django.contrib.auth.models import User
from wagtail.blocks import *
from wagtail.images.blocks import ImageChooserBlock

class WebPage(RoutablePageMixin, MetadataPageMixin, AbstractCarousel,AbstractHero, HitCountMixin, Page):
    template="webpage.html"
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    page_description = "Web Page e.g: Home,About & Contact "
    shortDesc = RichTextField(blank=True, null=True)
    menuName = models.CharField(
        max_length=100, null=True, blank=True, verbose_name="Menu Name")
    menuImageText = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Menu Icon Text")
    body = StreamField([
        ('section', coreSection()),
        ('row', rowSection()),
    ],blank=True,null=True,use_json_field=True)
    show_footer = models.BooleanField(blank=True, null=True, default=False)
    # modify your content_panels:
    # newsletter_template = "blogs/article_page_newsletter.html"

    content_panels = Page.content_panels+ [
        FieldPanel('menuImageText'),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('search_image',heading="Title Image"),
                FieldPanel('show_footer',widget=forms.CheckboxInput())
            ])
        ]),
        FieldPanel('shortDesc'),
        FieldPanel('body'),
    ]
    hero_panels=[
        FieldPanel('hero')
    ]
    banner_panel = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('bannerHeight'),
                FieldPanel('bannerWidth'),
                FieldPanel('bannerTimer'),
            ]),
            FieldPanel('bannercontent')
        ])
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(banner_panel, heading='Banner'),
        ObjectList(hero_panels, heading='Hero'),
        ObjectList(Page.promote_panels, heading='Promote'),
    ])
    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def get_absolute_url(self):
        return f'{self.url}'

    @re_path(r'^latest/$')
    def all_products(self, request):
        print("Product")
        events = ProductVariantPage.objects.filter(thumb=True)
        return render(request, "Pages/latest.html", {
            'total_products': events,
        })
        
        
    # @re_path(r'^<prod>/$')
    # def product(self, request):
    #     events = ProductVariantPage.objects.all()
    #     return render(request, "Product/CatList.html", {
    #         'total_products': events,
    #     })

class BlogIndexPage(MetadataPageMixin, AbstractCarousel,AbstractHero,RoutablePageMixin, Page):
    template="ListPage/blogs/blogs.html"
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    page_description = "Blogs Index Page Grid & List View"
    intro = RichTextField(blank=True)
    show_footer = models.BooleanField(blank=True, null=True, default=True)
    menuName = models.CharField(
        max_length=100, null=True, blank=True, verbose_name="Menu Name")
    menuImageText = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Icon Text")
    
    
    parent_page_type = [
        'blogs.WebPage',
    ]
    subpage_types = [
        'blogs.BlogPage',
    ]
    content_panels = Page.content_panels + [
        FieldPanel('menuImageText'),
        FieldPanel('intro'),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('search_image',heading="Title Image"),
                FieldPanel('show_footer',widget=forms.CheckboxInput())
            ])
        ]),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('bannerHeight'),
                FieldPanel('bannerWidth'),
                FieldPanel('bannerTimer'),
            ]),
            FieldPanel('bannercontent')
        ], heading="Banner"),
    ]
    hero_panels=[
        FieldPanel('hero')
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(hero_panels, heading='Hero'),
        ObjectList(Page.promote_panels, heading='Promote'),
    ])

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def mobile_breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs[3:]

    # def get_children(self,request,*args, **kwargs):
    #     qs = super().get_children()
    #     qs = qs.order_by('-last_published_at')
    #     return qs
    @path('all/')
    def all_blogs(self, request,*args, **kwargs):
        context = super().get_context(request, *args, **kwargs)
        all_posts = super().get_children().live().order_by('-first_published_at')
        paginator = Paginator(all_posts, 6)
        page = request.GET.get("page")
        try:
            posts = paginator.page(page)
        except PageNotAnInteger:
            posts = paginator.page(1)
        except EmptyPage:
            posts = paginator.page(paginator.num_pages)
        context["posts"] = posts
        return render(request,'ListPage/blogs/all_blogs.html',context)
    
    def get_absolute_url(self):
        return f'{self.url}'
    
    def get_context(self, request, *args, **kwargs):
        context = super().get_context(request, *args, **kwargs)
        all_posts = super().get_children().live().order_by('-first_published_at')
        paginator = Paginator(all_posts, 6)
        page = request.GET.get("page")
        try:
            posts = paginator.page(page)
        except PageNotAnInteger:
            posts = paginator.page(1)
        except EmptyPage:
            posts = paginator.page(paginator.num_pages)
        context["posts"] = posts
        context["recent"] = BlogPage.objects.all()[:10]
        return context

class BlogPageTag(ItemBase):
    tag = models.ForeignKey(
        CustomTag, related_name="tagged_blogs", on_delete=models.CASCADE
    )
    content_object = ParentalKey(
        'BlogPage',
        related_name='tagged_items',
        on_delete=models.CASCADE
    )

class BlogPage(RoutablePageMixin,MetadataPageMixin, AbstractCarousel, Page, HitCountMixin):
    template="DetailPage/blogs/blog.html"
    page_description = "Blog Detail Page"
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    customTitle = RichTextField(blank=True, null=True, verbose_name="Custom Title")
    fgColor = ColorField(
        default="#000000", verbose_name="FG", blank=True, null=True)
    bgColor = ColorField(
        default="#FFFFFF", verbose_name="BG", blank=True, null=True)
    bgImage = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.SET_NULL, related_name='+', blank=True, null=True, verbose_name="BG Image"
    )
    bgGradient = models.TextField(blank=True, null=True, verbose_name="BG Gradient")
    
    highlights = StreamField([ 
        ('title', HeadingBlock(required=False)),
        ('body', StreamBlock([
            ('content', StructBlock([
                ('color', NativeColorBlock(default="#CC0C39",required=False)),
                ('list', ListBlock(CharBlock()))
                ])),
            ]
        )),
    ], blank=True, null=True, use_json_field=True)
    cat = models.ForeignKey(
        Category, on_delete=models.SET_NULL, related_name='blogs', blank=True, null=True, verbose_name="Category"
    )
    tags = ClusterTaggableManager(through=BlogPageTag, blank=True)
    shortDesc = RichTextField(
        blank=True, null=True, verbose_name="Short Description")
    body = StreamField([
        ('section', SectionBlock()),
        ('row', RowBlock()),
    ], blank=True, null=True, use_json_field=True)
    show_author = models.BooleanField(blank=True, null=True, default=True)
    srcTitle = models.CharField(
        max_length=150, blank=True, null=True, verbose_name="Title")
    srcURL = models.URLField(blank=True, null=True, verbose_name="URL")
    readTime = models.IntegerField(
        default=2, blank=True, null=True, verbose_name="Read Time (minutes)")
    
    content_panels = Page.content_panels + [
        FieldPanel('customTitle'),
        MultiFieldPanel([
            FieldRowPanel([
                NativeColorPanel('fgColor'),
                NativeColorPanel('bgColor'),
                FieldPanel('bgImage'),
            ]),
            FieldPanel('bgGradient'),
        ],heading="Title Theme"),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('cat'),
                FieldPanel('show_author',widget=forms.CheckboxInput()),
            ]),
            FieldRowPanel([
                FieldPanel('search_image',heading="Title Image"),
                FieldPanel('readTime'),
            ])
        ]),
        FieldPanel('shortDesc'),
        FieldPanel('tags'),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('srcTitle'),
                FieldPanel('srcURL')
            ])
        ],heading="Source"),
        FieldPanel('highlights'),
        FieldPanel('body'),
    ]
    app_panels=[
        InlinePanel('blog_app_items')
    ]
    social_panels=[
        InlinePanel('blog_social')
    ]
    product_panels=[
        InlinePanel('blog_product',label="Product")
    ]
    banner_panels=[
        
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(social_panels, heading='Social'),
        ObjectList(app_panels, heading='App'),
        ObjectList(product_panels, heading='Products'),
        ObjectList(Page.promote_panels, heading='Promote'),
    ])

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def mobile_breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs[1:]

    def get_absolute_url(self):
        return f'{self.url}'
    def get_context(self, request, *args, **kwargs):
        context = super().get_context(request, *args, **kwargs)
        prodAttrs=[]
        similarProducts=[]
        products=self.blog_product.all()
        if self.blog_product.all():
            for rel in self.blog_product.all():
                if rel.product.content_type.model == "productpage":
                    for pv in rel.product.get_children().live():
                        if pv.specific.thumb:
                            similarProducts.append(ProductVariantPage.objects.get(title=pv.title))
            for pa in similarProducts:
                for attr in ProductVariantAttributeValue.objects.filter(product=pa):
                    if attr.attribute.attr:
                        prodAttrs.append(attr.attribute.attr.name)
        context['prodList']=list(set(similarProducts))
        context['similarAttr']=list(set(prodAttrs))
        
        if request.user.is_authenticated:
            if BlogBookmark.objects.filter(user=request.user,blog=self).exists():
                bookmark=True
            else:
                bookmark=False
        else:
            bookmark=False
        context['is_bookmarked']=bookmark
        return context
    
    @path('add-comment/')
    def add_comment(self, request):
        print("Req=",request.POST)
        if request.POST['comment']:
            BlogComment(user=request.user,blog=self,comment=request.POST['comment']).save()
        return HttpResponseRedirect(self.url)
    
    @path('add-bookmark/')
    def add_comment(self, request):
        if BlogBookmark.objects.filter(user=request.user,blog=self).exists():
            BlogBookmark.objects.get(user=request.user,blog=self).delete()
        else:
            print("Self=",request.user)
            BlogBookmark.objects.create(user=request.user,blog=self)
        return HttpResponseRedirect(self.url)

    class Meta:
        ordering=["-id"]
        verbose_name = "Blog"

class BlogSocial(Orderable): 
    blog = ParentalKey(BlogPage, related_name="blog_social")
    slp = models.ForeignKey(
        SocialPlatform, on_delete=models.SET_NULL, null=True,verbose_name="Social Platform")
    url = models.URLField(blank=True, null=True)
class BlogAppItems(Orderable):
    blog = ParentalKey(BlogPage, related_name="blog_app_items")
    item = models.ForeignKey(
        AppItems,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )
    panels=[
        SubcategoryChooser(
            "item",
            category=App,
            subcategory=AppItems,
            subcategory_related_name="app_items",
            search_text="Search items"
        ),
        
    ]
class BlogProduct(Orderable):
    blog = ParentalKey(BlogPage, related_name="blog_product")
    product=models.ForeignKey(ProductPage,on_delete=models.CASCADE,verbose_name="")

@register_snippet
class BlogBookmark(models.Model):
    user=models.ForeignKey(User,on_delete=models.CASCADE,related_name="bookmark_blogs")
    blog=models.ForeignKey(BlogPage,on_delete=models.CASCADE)

class BlogComment(models.Model):
    user=models.ForeignKey(User,on_delete=models.CASCADE,related_name="coment_blogs")
    blog=models.ForeignKey(BlogPage,on_delete=models.CASCADE,related_name="comments")
    comment=models.TextField()
    date= models.DateTimeField(auto_now_add=True)
    


class NewsIndexPage(MetadataPageMixin, AbstractCarousel, Page):
    template="ListPage/blogs/NewsList.html"
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    page_description = "News Index Page Grid & List View"
    intro = RichTextField(blank=True)
    show_footer = models.BooleanField(blank=True, null=True, default=True)
    menuName = models.CharField(
        max_length=100, null=True, blank=True, verbose_name="Menu Name")
    menuImageText = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Icon Text")
    subpage_types = [
        'blogs.NewsPage',
    ]
    content_panels = Page.content_panels + [
        MultiFieldPanel([
           FieldRowPanel([
               FieldPanel('search_image',heading="Title Image"),
               FieldPanel('show_footer',widget=forms.CheckboxInput()),
               FieldPanel('menuImageText'),
           ]) 
        ]),
        FieldPanel('intro'),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('bannerHeight'),
                FieldPanel('bannerWidth'),
                FieldPanel('bannerTimer'),
            ]),
            FieldPanel('bannercontent')
        ], heading="Banner"),
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(Page.promote_panels, heading='Promote'),
    ])
   
    def get_children(self):
        qs = super().get_children()
        qs = qs.order_by('-first_published_at')
        return qs.live()

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def get_absolute_url(self):
        return f'{self.url}'

     
    def get_context(self, request,*args, **kwargs):
        context = super().get_context(request, *args, **kwargs)
        trends=[]
        filterObj=''
        title=''
        q = request.GET.get('q')
        if q:
            title=q
            if Category.objects.filter(slug=q).exists():
                filterObj=NewsPage.objects.filter(cat__slug=q)
                # title=Category.objects.get(slug=q).name
            else:
                # title=q
                tg=[]
                tg.append(q)
                filterObj=NewsPage.objects.filter(tags__slug__in=tg)
            # pass
        for news in NewsPage.objects.all():
            if HitCount.objects.filter(object_pk = news.id).exists():
                trends.append(news)
        context['trends'] = trends[:10]
        context['filterObj'] = filterObj
        context['title'] = title
        
        all_posts = super().get_children().live().order_by('-last_published_at')
        paginator = Paginator(all_posts, 5)
        page = request.GET.get("page")
        try:
            posts = paginator.page(page)
        except PageNotAnInteger:
            posts = paginator.page(1)
        except EmptyPage:
            posts = paginator.page(paginator.num_pages)
        context["posts"] = posts
        
        return context

class NewsPageTag(ItemBase):
    tag = models.ForeignKey(
        CustomTag, related_name="tagged_news", on_delete=models.CASCADE
    )
    content_object = ParentalKey(
        'NewsPage',
        related_name='tagged_items',
        on_delete=models.CASCADE
    )

class NewsPage(MetadataPageMixin, AbstractCarousel, Page, AbstractCustomColor, HitCountMixin):
    template="DetailPage/blogs/News.html"
    page_description = "News Detail Page"
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    customTitle = RichTextField(
        blank=True, null=True, verbose_name="Custom Title")
    cat = models.ForeignKey(
        Category, on_delete=models.SET_NULL, related_name='cat_news', blank=True, null=True, verbose_name="Category"
    )
    product = models.ForeignKey(
        ProductPage, on_delete=models.SET_NULL, related_name='pr_news', blank=True, null=True, verbose_name="Product"
    )
    srcTitle = models.CharField(
        max_length=150, blank=True, null=True, verbose_name="Title")
    srcURL = models.URLField(blank=True, null=True, verbose_name="URL")
    show_author = models.BooleanField(blank=True, null=True, default=False)
    readTime = models.IntegerField(
        default=2, blank=True, null=True, verbose_name="Read Time (minutes)")
    rtColor = ColorField(
        default="#23A10F", verbose_name="FG", blank=True, null=True)
    shortDesc = RichTextField(
        blank=True, null=True, verbose_name="Short Description")
    tags = ClusterTaggableManager(through=NewsPageTag, blank=True)
    body = StreamField([
        ('section', SectionBlock()),
        ('row', RowBlock()),
    ], blank=True, null=True, use_json_field=True)
    
    highlights = StreamField([ 
        ('title', HeadingBlock(required=False)),
        ('body', StreamBlock([
            ('content', StructBlock([
                ('color', NativeColorBlock(default="#CC0C39",required=False)),
                ('list', ListBlock(CharBlock()))
                ])),
            ]
        )),
    ], blank=True, null=True, use_json_field=True)
    
    content_panels = Page.content_panels + [
        FieldPanel('customTitle'),
        # FieldPanel('go_live_at'),
        # FieldPanel('expire_at'),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('search_image', heading="Title Image"),
                FieldPanel('show_author',widget=forms.CheckboxInput()),
            ])
        ]),
        FieldPanel('shortDesc'),
        
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('cat'),
                FieldPanel('product'),
            ]),
        ]),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('srcTitle'),
                FieldPanel('srcURL'),
            ]),
            FieldRowPanel([
                FieldPanel('readTime'),
                NativeColorPanel('rtColor'),
            ]),
        ], heading="Source & Read Time"),
       
        FieldPanel('tags'),
        FieldPanel('highlights'),        
        FieldPanel('body'),
    ]
    related_panels = [
        InlinePanel('news_products')
    ]
    social_panels = [InlinePanel('news_social')]
    banner_panel = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('bannerHeight'),
                FieldPanel('bannerWidth'),
                FieldPanel('bannerTimer'),
            ]),
            FieldPanel('bannercontent')
        ])
    ]
    app_panels=[
        InlinePanel('news_app_items')
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(banner_panel, heading='Banner'),
        ObjectList(social_panels, heading='Social'),
        ObjectList(Page.promote_panels, heading='Promote'),
        ObjectList(related_panels, heading='Related Products'),
        ObjectList(app_panels, heading='App'),
    ])
    search_fields = Page.search_fields + [
        index.SearchField('title'),
        index.SearchField('slug'),
        index.SearchField('customTitle'),
        index.SearchField('highlights'),
        index.SearchField('shortDesc'),
        index.SearchField('tags'),
        index.SearchField('cat'),
        index.SearchField('product'),

        index.FilterField('title'),
        index.FilterField('slug'),
        index.FilterField('customTitle'),
        index.FilterField('highlights'),
        index.FilterField('shortDesc'),
        index.FilterField('tags'),
        index.FilterField('cat'),
        index.FilterField('product'),
    ]

    def get_context(self, request):
        context = super().get_context(request)
        context['related_news'] = NewsPage.objects.filter(
            cat=self.cat).order_by("-first_published_at")[:10]
        # context['products'] = self.cat.cat_news.all()[:10]
        context['trends'] = "Coming Soon"
        return context

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def mobile_breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs[4:]

    def get_absolute_url(self):
        return f'{self.url}'
    
    class Meta:
        verbose_name = "News"

class NewsSocial(Orderable): 
    news = ParentalKey(NewsPage, related_name="news_social")
    slp = models.ForeignKey(
        SocialPlatform, on_delete=models.SET_NULL, null=True,verbose_name="Social Platform")
    url = models.URLField(blank=True, null=True)
class NewsProduct(Orderable):
    news = ParentalKey(NewsPage, related_name="news_products")
    product = models.ForeignKey(
        ProductPage, on_delete=models.SET_NULL, null=True)
class NewsAppItems(Orderable):
    blog = ParentalKey(NewsPage, related_name="news_app_items")
    item = models.ForeignKey(
        AppItems,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )
    panels=[
        SubcategoryChooser(
            "item",
            category=App,
            subcategory=AppItems,
            subcategory_related_name="app_items",
            search_text="Search items"
        ),
        
    ]



class TutorialIndexPage(MetadataPageMixin, AbstractCarousel, Page):
    template="ListPage/blogs/tutorials.html"
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    page_description = "Tutorial Index Page Grid & List View"
    intro = RichTextField(blank=True)
    show_footer = models.BooleanField(blank=True, null=True, default=True)
    menuName = models.CharField(
        max_length=100, null=True, blank=True, verbose_name="Menu Name")
    menuImageText = models.CharField(
        max_length=250, blank=True, null=True, verbose_name="Icon Text")
    subpage_types = [
        'blogs.TutorialPage', 
    ]
    content_panels = Page.content_panels + [
        MultiFieldPanel([
           FieldRowPanel([
               FieldPanel('search_image',heading="Title Image"),
               FieldPanel('show_footer',widget=forms.CheckboxInput()),
               FieldPanel('menuImageText'),
           ]) 
        ]),
        FieldPanel('intro'),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('bannerHeight'),
                FieldPanel('bannerWidth'),
                FieldPanel('bannerTimer'),
            ]),
            FieldPanel('bannercontent')
        ], heading="Banner"),
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(Page.promote_panels, heading='Promote'),
    ])

    def get_children(self):
        qs = super().get_children()
        qs = qs.order_by('-first_published_at')
        return qs.live()

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def get_absolute_url(self):
        return f'{self.url}'

    def get_context(self, request, *args, **kwargs):
        context = super().get_context(request, *args, **kwargs)
        all_posts = super().get_children().live().order_by('-last_published_at')
        paginator = Paginator(all_posts, 6)
        page = request.GET.get("page")
        try:
            posts = paginator.page(page)
        except PageNotAnInteger:
            posts = paginator.page(1)
        except EmptyPage:
            posts = paginator.page(paginator.num_pages)
        context["posts"] = posts
        return context
    
class TutorialPageTag(ItemBase):
    tag = models.ForeignKey(
        CustomTag, related_name="tagged_tutorial", on_delete=models.CASCADE
    )
    content_object = ParentalKey(
        'TutorialPage',
        related_name='tagged_items',
        on_delete=models.CASCADE
    )

class TutorialPage(RoutablePageMixin,MetadataPageMixin, AbstractCarousel, Page, HitCountMixin):
    template="DetailPage/blogs/tutorial.html"
    page_description = "Tutorial Detail Page"
    objects = PageManager()  # needed, so _default_manager isn't the trash manager
    objects_excluding_trash = TrashManager()
    customTitle = RichTextField(
        blank=True, null=True, verbose_name="Custom Title")
    fgColor = ColorField(
        default="#000000", verbose_name="FG", blank=True, null=True)
    bgColor = ColorField(
        default="#FFFFFF", verbose_name="BG", blank=True, null=True)
    bgImage = models.ForeignKey(
        'wagtailimages.Image', on_delete=models.SET_NULL, related_name='+', blank=True, null=True, verbose_name="BG Image"
    )
    bgGradient = models.TextField(blank=True, null=True, verbose_name="BG Gradient")
    
    cat = models.ForeignKey(
        Category, on_delete=models.SET_NULL, related_name='cat_tutorials', blank=True, null=True, verbose_name="Category"
    )
    srcTitle = models.CharField(
        max_length=150, blank=True, null=True, verbose_name="Title")
    srcURL = models.URLField(blank=True, null=True, verbose_name="URL")
    show_author = models.BooleanField(blank=True, null=True, default=False)
    readTime = models.IntegerField(
        default=2, blank=True, null=True, verbose_name="Read Time (minutes)")
    rtColor = ColorField(
        default="#23A10F", verbose_name="FG", blank=True, null=True)
    shortDesc = RichTextField(
        blank=True, null=True, verbose_name="Short Description")
    tags = ClusterTaggableManager(through=TutorialPageTag, blank=True)
    body = StreamField([
        ('section', SectionBlock()),
        ('row', RowBlock()),
    ], blank=True, null=True, use_json_field=True)
    
    highlights = StreamField([ 
        ('title', HeadingBlock(required=False)),
        ('body', StreamBlock([
            ('content', StructBlock([
                ('color', NativeColorBlock(default="#CC0C39",required=False)),
                ('list', ListBlock(CharBlock()))
                ])),
            ]
        )),
    ], blank=True, null=True, use_json_field=True)
    

    content_panels = Page.content_panels + [
        FieldPanel('customTitle'),
        MultiFieldPanel([
            FieldRowPanel([
                NativeColorPanel('fgColor'),
                NativeColorPanel('bgColor'),
                FieldPanel('bgImage'),
            ]),
            FieldPanel('bgGradient'),
        ],heading="Title Theme"),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('search_image', heading="Title Image"),
                FieldPanel('show_author',widget=forms.CheckboxInput()),
            ])
        ]),
        FieldPanel('shortDesc'),
        
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('cat'),
            ]),
        ]),
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('srcTitle'),
                FieldPanel('srcURL'),
            ]),
            FieldRowPanel([
                FieldPanel('readTime'),
                NativeColorPanel('rtColor'),
            ]),
        ], heading="Source & Read Time"),
       
        FieldPanel('tags'),
        FieldPanel('highlights'),        
        FieldPanel('body'),
    ]
    social_panels = [InlinePanel('tutorial_social')]
    banner_panel = [
        MultiFieldPanel([
            FieldRowPanel([
                FieldPanel('bannerHeight'),
                FieldPanel('bannerWidth'),
                FieldPanel('bannerTimer'),
            ]),
            FieldPanel('bannercontent')
        ])
    ]
    app_panels=[
        InlinePanel('tutorial_app_items')
    ]
    edit_handler = TabbedInterface([
        ObjectList(content_panels, heading='Content'),
        ObjectList(banner_panel, heading='Banner'),
        ObjectList(social_panels, heading='Social'),
        ObjectList(Page.promote_panels, heading='Promote'),
        ObjectList(app_panels, heading='App'),
    ])
    search_fields = Page.search_fields + [
        index.SearchField('title'),
        index.SearchField('slug'),
        index.SearchField('customTitle'),
        index.SearchField('highlights'),
        index.SearchField('shortDesc'),
        index.SearchField('tags'),
        index.SearchField('cat'),

        index.FilterField('title'),
        index.FilterField('slug'),
        index.FilterField('customTitle'),
        index.FilterField('highlights'),
        index.FilterField('shortDesc'),
        index.FilterField('tags'),
        index.FilterField('cat'),
    ]

    def breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs

    def mobile_breadcrumb_list(self):
        breadcrumbs = []
        page = self
        while page.get_parent() is not None:
            if page.is_root():
                break
            page = page.get_parent()
            breadcrumbs.append(page)
        breadcrumbs.reverse()
        return breadcrumbs[4:]

    def get_absolute_url(self):
        return f'{self.url}'
    
    def get_context(self, request, *args, **kwargs):
        context = super().get_context(request, *args, **kwargs)
        if request.user.is_authenticated:
            if TutorialBookmark.objects.filter(user=request.user,tutorial=self).exists():
                bookmark=True
            else:
                bookmark=False
        else:
            bookmark=False
        context['is_bookmarked']=bookmark
        return context
    
    @path('add-comment/')
    def add_comment(self, request):
        print("Req=",request.POST)
        if request.POST['comment']:
            TutorialComment(user=request.user,tutorial=self,comment=request.POST['comment']).save()
        return HttpResponseRedirect(self.url)

    @path('add-bookmark/')
    def add_comment(self, request):
        if TutorialBookmark.objects.filter(user=request.user,tutorial=self).exists():
            TutorialBookmark.objects.get(user=request.user,tutorial=self).delete()
        else:
            TutorialBookmark.objects.create(user=request.user,tutorial=self)
        return HttpResponseRedirect(self.url)

    class Meta:
        verbose_name = "Tutorial"

class TutorialAppItems(Orderable):
    blog = ParentalKey(TutorialPage, related_name="tutorial_app_items")
    item = models.ForeignKey(
        AppItems,
        on_delete=models.SET_NULL,
        null=True,
        blank=True,
    )
    panels=[
        SubcategoryChooser(
            "item",
            category=App,
            subcategory=AppItems,
            subcategory_related_name="app_items",
            search_text="Search items"
        ),
        
    ]
class TutorialSocial(Orderable): 
    tutorial = ParentalKey(TutorialPage, related_name="tutorial_social")
    slp = models.ForeignKey(
        SocialPlatform, on_delete=models.SET_NULL, null=True,verbose_name="Social Platform")
    url = models.URLField(blank=True, null=True)

class TutorialBookmark(models.Model):
    user=models.ForeignKey(User,on_delete=models.CASCADE,related_name="bookmark_tutorials")
    tutorial=models.ForeignKey(TutorialPage,on_delete=models.CASCADE)


class TutorialComment(models.Model):
    user=models.ForeignKey(User,on_delete=models.CASCADE,related_name="comment_tutorials")
    tutorial=models.ForeignKey(TutorialPage,on_delete=models.CASCADE,related_name="comments")
    comment=models.TextField()
    date= models.DateTimeField(auto_now_add=True)
    
